//
//  CroppingNativeView.swift
//  scanbot_sdk
//
//  Created by Serhii Chaban on 04.01.23.
//

import Foundation
import Flutter
import ScanbotSDK
class CroppingNativeView : NSObject, FlutterPlatformView, CallErrorHandler, SBSDKImageEditingViewControllerDelegate {
    private var viewController: SBSDKImageEditingViewController?
    
    private let kViewControllerAppearDelay = 0.2
    private var containerView: UIView
    private let configuration : [String: Any?]?
    private var croppingMethodChannel: FlutterMethodChannel?
    private var processor: SBSDKDocumentDetector?
    private var currentPage: SBSDKUIPage?
    init(
        frame: CGRect,
        viewIdentifier viewId: Int64,
        arguments args: Any?,
        binaryMessenger messenger: FlutterBinaryMessenger
    ) {
        
        croppingMethodChannel = FlutterMethodChannel(name: "scanbot_sdk_cropping_\(viewId)", binaryMessenger:  messenger)
        
        containerView = UIView()
        configuration = args as? [String:Any?]
        
        processor = SBSDKDocumentDetector(mode: SBSDKDocumentDetector.defaultDetectorMode())
        
        super.init()
        
        self.createNativeView(view: self.containerView)
        
        self.croppingMethodChannel?.setMethodCallHandler({
            (call: FlutterMethodCall, result: @escaping FlutterResult) -> Void in
            self.handleCameraCall(call: call, result: result)
        })
    }
    
    func view() -> UIView {
        return containerView
    }
    
    func createNativeView(view _view: UIView){
        if(!ScanbotSDK.isLicenseValid()){
            DispatchQueue.main.async { [weak self] in
                guard let callHandler = self?.croppingMethodChannel else {
                    fatalError("callHandler cannot be nil")
                }
                self?.sendLicenseErrorToDetector(callHandler)
            }
            return
        }
        
        let rootViewController = UIViewController()
        rootViewController.view = _view
        rootViewController.view.backgroundColor = UIColor.clear
        viewController = SBSDKImageEditingViewController(parentViewController: rootViewController, containerView: _view)
        viewController?.delegate = self
        
        if let arguments = configuration {
            setInitialConfiguration(arguments: arguments)
        }
        
    }
    
    func setInitialConfiguration(arguments:[String:Any?]) {
        guard let viewController = viewController else {
            return
        }
        guard let page = SBSDKUIPage(from: arguments as [String : Any]) else {return }
        currentPage = page
        guard let image = page.originalPreviewImage() else {return }
        viewController.view.backgroundColor = UIColor.clear
        
        if let strokeWidth =  arguments["edgeLineWidth"] as? Double {
            viewController.edgeLineWidth = strokeWidth
        }
        
        if let hex = arguments["edgeColor"] as? String {
            viewController.edgeColor = UIColor(hexString: hex)
        }
        
        if let hex = arguments["anchorPointsColor"] as? String {
            viewController.anchorPointsColor = UIColor(hexString: hex)
        }
        
        if let hex = arguments["edgeColorOnLine"] as? String {
            viewController.magneticEdgeColor = UIColor(hexString: hex)
        }
        
        if let insets = arguments["borderInsets"] as? [String : Any] {
            if let insets = UIEdgeInsets(from: insets) {
                viewController.contentInsets = insets
            }
        }
        viewController.processingQueue = DispatchQueue.main
        viewController.polygon = page.polygon ?? SBSDKPolygon()
        viewController.image =  image                          
    }
    
    
    func handleCameraCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        guard let viewController = viewController,
              let page = currentPage else {
            returnGuardError(result: result)
            return
        }
        
        switch call.method {
        case "rotateCW" :
            viewController.rotateInputImageClockwise(true, animated: true)
            let size = viewController.image.size
            page.polygon?.rotateCW(UInt(1), with: size)
        case "rotateCCW" :
            viewController.rotateInputImageClockwise(false, animated: true)
            let size = viewController.image.size
            page.polygon?.rotateCCW(UInt(1), with: size)
        case "getEditedPolygon" :
            guard let polygon = viewController.polygon else {
                returnGuardError(result: result)
                return
            }
            let rotation = viewController.rotations
            result(getJsonForPolygonData(polygon:polygon, rotation:rotation).encodedString)
            return
        case "reset" :
            let resetPolygon = SBSDKPolygon.init()
            viewController.polygon = resetPolygon

        case "detect" :
                let image = viewController.image
                                
                let result = processor?.detectDocumentPolygon(on: image,
                                                              visibleImageRect: CGRect.zero,
                                                              smoothingEnabled: false,
                                                              useLiveDetectionParameters: false)
                
                viewController.polygon = result?.polygon
        case "dispose" : detach()
        default:
            self.returnUnknownCallError(call: call, result: result)
            return
        }
        resurnSuccess(result: result)
    }
    
    func getJsonForPolygonData(polygon: SBSDKPolygon, rotation: Int)-> [String: Any]{
        var croppingResult: [String: Any] = [:]
        croppingResult["polygon"] = polygon.encodedDictionary
        croppingResult["rotationTimesCw"] = rotation
        return croppingResult
    }
    
    func resurnSuccess(result: @escaping FlutterResult){
        result("success")
    }
    
    func returnGuardError(result: @escaping FlutterResult){
        result(result(FlutterError(code: "1005",
                                   message: "ERROR_iOS: can apply methode due to the reason: one of the parameters is null",
                                   details: nil)))
    }
    
    func detach(){
        croppingMethodChannel?.setMethodCallHandler(nil)
        viewController?.dismiss(animated: true)
        viewController = nil
    }
    
    
    func imageEditingViewControllerDidChangePolygon(_ editingViewController: SBSDKImageEditingViewController) {
        if editingViewController.polygon?.isSimilar(to: SBSDKPolygon()) == false {
            
        }
    }
    
    func sendLicenseErrorToDetector(_ callHandler: FlutterMethodChannel) {
        let isValid = ScanbotSDK.isLicenseValid()
        let status = ScanbotSDK.licenseStatus()
        let expirationDate = ScanbotSDK.licenseExpirationDate()
        
        var dictionary: [String: Any?] = ["isLicenseValid": isValid, "status": status.stringName]
        if let date = expirationDate {
            let formatter = DateFormatter()
            formatter.timeZone = TimeZone(abbreviation: "UTC")
            formatter.dateFormat = "YYYY-MM-dd HH:mm:ss"
            formatter.locale = Locale(identifier: "en_EN")
            let dateString = formatter.string(from: date)
            dictionary["expirationDate"] = dateString
        }
        dictionary["message"] = "license is no more valid for this feature"
        callHandler.invokeMethod("onLicenseError", arguments: dictionary.encodedString)
    }
}
