//
//  MedicalCertificate.swift
//  scanbot_sdk
//
//  Created by Serhii Chaban on 14.02.23.
//

import Foundation
import ScanbotSDK
import UIKit

class MedicalCertificateScanner: NSObject, SBSDKScanner, SBSDKMedicalCertificateScannerViewControllerDelegate, CallErrorHandler  {
    
    private var viewController: SBSDKMedicalCertificateScannerViewController?
    
    private var cameraMethodChannel: FlutterMethodChannel?
    private let detectorChannel: FlutterMethodChannel?
    private let finderChannel: FlutterMethodChannel?
    private var configuration: [String:Any?]
   // private var imageGenerationType: SBSDKBarcodeImageGenerationType = SBSDKBarcodeImageGenerationType.none
    private var cachedResult: SBSDKMedicalCertificateRecognizerResult? = nil
    var shouldRunDetection: Bool = true
    var shouldPauseDetectionOnResultReturn: Bool = false
    var shouldRecognizePatientInfo: Bool = true
    var shouldRecognizeBarcode: Bool = true
    var shouldDetectMcOnSnappedImage: Bool = false
    var shouldReturnCroppedImage: Bool = false
 
    /// Recognition of Patient Information block. [true] if enabled
    var recognizePatientInfo: Bool = true

    ///Reading data from barcode, if it's present on document, could make data recognition more accurate. [true] if enabled.
    var recognizeBarcode: Bool = true

    ///Make high-res snapshot of the image after first successfull medical certificate detection and recognise medical certificate fields on it
    var captureHighResolutionImage: Bool = false

    ///return url of the cropped image
    var returnCroppedImage: Bool = false

    init(
        cameraMethodChannel: FlutterMethodChannel?,
        detectorChannel: FlutterMethodChannel?,
        finderChannel: FlutterMethodChannel?,
        configuration: [String:Any?]
    ){
        self.cameraMethodChannel = cameraMethodChannel
        self.detectorChannel = detectorChannel
        self.finderChannel = finderChannel
        self.configuration = configuration
    }
    
    func createNativeView(view _view: UIView){
        
        if(!ScanbotSDK.isLicenseValid()){
            DispatchQueue.main.async { [weak self] in
                guard let callHandler = self?.detectorChannel else {
                    fatalError("callHandler cannot be nil")
                }
                self?.sendLicenseErrorToDetector(callHandler)
            }
            return
        }
        
        let rootViewController = UIViewController()
        rootViewController.view = _view
        viewController = SBSDKMedicalCertificateScannerViewController(parentViewController: rootViewController,
                                                                      parentView: nil,
                                                          delegate: self)
        let _ = viewController?.view
        setMcScannerConfiguration(configuration:configuration)
        cameraMethodChannel?.invokeMethod(MethodChannelConstants.kOnCameraOpenMethod, arguments:"")
    }
    
    func attach(view _view: UIView) {
        createNativeView(view: _view)
        self.detectorChannel?.setMethodCallHandler({
            (call: FlutterMethodCall, result: @escaping FlutterResult) -> Void in
            self.handleDetectorCall(call: call, result: result)
        })
    }
    
    func setFlash(enabled: Bool) {
        viewController?.isFlashLightEnabled = enabled
    }
    
    func isFlashAvailable() -> Bool {
        return viewController?.isFlashLightAvailable ?? false
    }
    
    func detach(){
        detectorChannel?.setMethodCallHandler(nil)
        viewController?.dismiss(animated: true)
        viewController = nil
    }
    
    
    func medicalCertificateScannerViewControllerShouldRecognize(_ controller: SBSDKMedicalCertificateScannerViewController) -> Bool {
        return shouldRunDetection && !shouldPauseDetectionOnResultReturn
    }

    func medicalCertificateScannerViewController(_ controller: SBSDKMedicalCertificateScannerViewController, didRecognizeMedicalCertificate result: SBSDKMedicalCertificateRecognizerResult) {
        shouldPauseDetectionOnResultReturn = true
        var inProcessing = true
        defer {
            inProcessing = false
            shouldPauseDetectionOnResultReturn = false
        }
        cameraMethodChannel?.invokeMethod("onShowHeavyProcessing", arguments: true)
        cachedResult = result
        var imageUri : URL? = nil
        if(captureHighResolutionImage){
                if let image = result.image as? UIImage {
                    guard let uuid = SBSDKUIPageFileStorage.default().add(image) else { return  }
                    imageUri = SBSDKUIPageFileStorage.default().imageURL(withPageFileID: uuid, pageFileType: .original)
                }

        }
        sendDetectedResult(result, imageUri?.standardizedFileURL.absoluteString)
        cameraMethodChannel?.invokeMethod("onShowHeavyProcessing", arguments: false)
    }
    
    
    func didChangeViewFinderRect(rect: CGRect) {
        let dictionary: [String: Any?] = ["left": rect.minX,
                                          "top": rect.minY,
                                          "right": rect.maxX,
                                          "bottom": rect.maxY,]
        
        finderChannel?.invokeMethod("finderSizeChanged", arguments: dictionary)
    }
    
    fileprivate func sendDetectedResult(_ result: SBSDKMedicalCertificateRecognizerResult,_ imageImageURI: String?) {
        var resultDict =  result.toSdkJson()
        resultDict["croppedDocumentURI"] = imageImageURI

        DispatchQueue.main.async { [weak self] in
            guard let callHandler = self?.detectorChannel else {
                fatalError("callHandler cannot be nil")
            }
            callHandler.invokeMethod("onMedicalCertificateDetected", arguments: resultDict.encodedString)
        }
    }
    
    
    fileprivate func configureFinderView(_ configuration: [String : Any?]?) {
        if let finderConfiguration: [String:Any] = configuration?["finder"] as? [String : Any] {
            let finderConfig = SBSDKBaseScannerViewFinderConfiguration.default()
            finderConfig.backgroundColor = UIColor(hexString: "#00000000")
            finderConfig.lineColor = UIColor(hexString: "#00000000")
            finderConfig.isViewFinderEnabled = true
            
            
            if
                let finderAspectRatio = finderConfiguration["finderAspectRatio"] as? [String:Any],
                let width = finderAspectRatio["width"] as? Double,
                let height = finderAspectRatio["height"] as? Double
            {
                finderConfig.aspectRatio = SBSDKAspectRatio(width: width, andHeight: height)
            }
            
            if let insets = finderConfiguration["finderInsets"] as? [String : Any] {
                if let left = insets["left"] as? CGFloat {
                    finderConfig.minimumInset.left = left
                }
                
                if let top = insets["top"] as? CGFloat {
                    finderConfig.minimumInset.top = top
                }
                
                if let right = insets["right"] as? CGFloat {
                    finderConfig.minimumInset.right = right
                }
                
                if let bottom = insets["bottom"] as? CGFloat {
                    finderConfig.minimumInset.bottom = bottom
                }
            }
            viewController?.viewFinderConfiguration = finderConfig
        }
    }
    
    func setMcScannerConfiguration(configuration:[String:Any?]?) {
        
        viewController?.viewFinderRectDidChangeHandler = {(finderRect:CGRect) -> Void in
            self.didChangeViewFinderRect(rect:finderRect)
        }
        
        if let flashEnabled = configuration?["flashEnabled"] as? Bool {
            viewController?.isFlashLightEnabled = flashEnabled
        }
        
        if let cameraZoomFactor = configuration?["cameraZoomFactor"] as? CGFloat {
            let zoomConfig = SBSDKBaseScannerZoomConfiguration.default()
            zoomConfig.initialZoomFactor = cameraZoomFactor
            viewController?.zoomConfiguration = zoomConfig
        }
        
        configureFinderView(configuration)
        if let scannerConfiguration: [String:Any] = configuration?["scannerConfiguration"] as? [String : Any] {
          
            if let detectMcOnSnappedImage = scannerConfiguration["captureHighResolutionImage"] as? Bool {
                viewController?.captureHighResolutionImage = detectMcOnSnappedImage
                self.captureHighResolutionImage = detectMcOnSnappedImage
            }
       
            if let recognizePatientInfo = scannerConfiguration["recognizePatientInfo"] as? Bool {
                viewController?.extractPatientInfo = recognizePatientInfo
                self.recognizePatientInfo = recognizePatientInfo
            }
            
            if let recognizeBarcode = scannerConfiguration["recognizeBarcode"] as? Bool {
                viewController?.extractBarcode = recognizeBarcode
                self.recognizeBarcode = recognizeBarcode
            }
        }
        
    }
    
    func stopPreview() {
        viewController?.freezeCamera()
    }
    
    func resumePreview() {
        viewController?.unfreezeCamera()
    }
    
    func handleDetectorCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "enable" : shouldRunDetection = true
        case "disable" : shouldRunDetection = false
        default:
            self.returnUnknownCallError(call: call, result: result)
            return
        }
        resurnSuccess(result: result) //always return some result to the flutter
    }
    
    func resurnSuccess(result: @escaping FlutterResult){
        result("success")
    }
}
