import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:json_annotation/json_annotation.dart';
import 'package:scanbot_sdk/json/common_data.dart' as common;

import '../json/common_data.dart';
import '../sdk_utils.dart';

part 'classical_cropping.g.dart';

class ScanbotCroppingWidget extends StatefulWidget {
  final common.Page page;

  /// Width of the cropping rectangle lines
  final double edgeLineWidth;

  /// Color of the cropping rectangle lines
  final Color edgeColor;

  /// Color of the cropping rectangle lines when they are snapped in place
  final Color edgeColorOnLine;

  /// Color of the cropping rectangle anchor points
  final Color anchorPointsColor;

  /// Distance from the edge of the widget to the cropped image
  final common.Insets? borderInsets;

  /// Triggers when the plugin takes long to process an operation eg. polygon detection
  final Function(bool)? onHeavyOperationProcessing;

  /// Triggers when the view is ready
  final Function(CroppingController) onViewReady;

  ScanbotCroppingWidget({
    Key? key,
    required this.page,
    required this.onViewReady,
    this.anchorPointsColor = Colors.red,
    this.edgeColor = Colors.white,
    this.edgeColorOnLine = Colors.green,
    this.edgeLineWidth = 5,
    this.onHeavyOperationProcessing,
    this.borderInsets,
  }) : super(key: key);

  @override
  _ScanbotCroppingWidgetState createState() => _ScanbotCroppingWidgetState();
}

class _ScanbotCroppingWidgetState extends State<ScanbotCroppingWidget> {
  @visibleForTesting
  static late String channelName;

  late MethodChannel _croppingChannel;

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return OrientationBuilder(
      builder: (context, orientation) {
        Map<String, dynamic> json = prepareConfigJson(orientation);

        if (defaultTargetPlatform == TargetPlatform.android) {
          return Stack(
            children: [
              AndroidView(
                key: Key("cropping_${orientation.name}"),
                viewType: "scanbot_sdk_cropping",
                layoutDirection: TextDirection.ltr,
                onPlatformViewCreated: _onPlatformViewCreated,
                creationParams: json,
                creationParamsCodec: const StandardMessageCodec(),
              ),
            ],
          );
        }
        if (defaultTargetPlatform == TargetPlatform.iOS) {
          return Stack(
            children: [
              UiKitView(
                // key: Key("cropping_${orientation.name}"),
                viewType: "scanbot_sdk_cropping",
                layoutDirection: TextDirection.ltr,
                onPlatformViewCreated: _onPlatformViewCreated,
                creationParams: json,
                creationParamsCodec: const StandardMessageCodec(),
              ),
            ],
          );
        }
        return Text(
            '$defaultTargetPlatform is not yet supported by the camera plugin');
      },
    );
  }

  Map<String, dynamic> prepareConfigJson(Orientation orientation) {
    var json = Map<String, dynamic>();
    json["layout_orientation"] = orientation.name;
    json["page"] = widget.page.toJson();
    json["edgeLineWidth"] = widget.edgeLineWidth;
    json["edgeColorOnLine"] = toColorString(widget.edgeColorOnLine);
    json["edgeColor"] = toColorString(widget.edgeColor);
    json["anchorPointsColor"] = toColorString(widget.anchorPointsColor);
    if (widget.borderInsets != null) {
      json["borderInsets"] = widget.borderInsets?.toJson();
    }
    return json;
  }

  void _onPlatformViewCreated(int id) {
    channelName = "scanbot_sdk_cropping_$id";
    _croppingChannel = MethodChannel(channelName);
    _croppingChannel.setMethodCallHandler(_resultHandler);
    widget.onViewReady(CroppingController(_croppingChannel));
  }

  Future<dynamic> _resultHandler(MethodCall methodCall) async {
    switch (methodCall.method) {
      case 'onShowHeavyProcessing':
        widget.onHeavyOperationProcessing?.call(methodCall.arguments as bool);
        break;
      default:
    }
    return;
  }

  @override
  void dispose() {
    super.dispose();
    _croppingChannel.invokeMethod("dispose");
  }
}

class CroppingController {
  MethodChannel _croppingChannel;

  CroppingController(this._croppingChannel);

  Future<void> rotateCw() async {
    return await _croppingChannel.invokeMethod("rotateCW");
  }

  Future<void> rotateCCW() async {
    return await _croppingChannel.invokeMethod("rotateCCW");
  }

  Future<void> reset() async {
    return await _croppingChannel.invokeMethod("reset");
  }

  Future<void> detect() async {
    return await _croppingChannel.invokeMethod("detect");
  }

  Future<CroppingResult> getPolygon() async {
    var json = await _croppingChannel.invokeMethod("getEditedPolygon");
    return CroppingResult.fromJson(jsonDecode(json));
  }
}

/// The Scanbot SDK provides a ready-to-use UI for document scanning and cropping.
///Both components use the notion of a 'page' as a data model for the scanning and cropping activities.
@JsonSerializable(
    includeIfNull: false, createFactory: true, createToJson: false)
class CroppingResult {
  ///  Cropping bounds for document after editing
  List<PolygonPoint> polygon;

  /// Rotation of the image after editing
  int rotationTimesCw;

  CroppingResult(this.rotationTimesCw, this.polygon);

  factory CroppingResult.fromJson(Map<String, dynamic> json) =>
      _$CroppingResultFromJson(json);
}
